@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """Interfaces for handling BIDS-like neuroimaging structures."""
 from collections import defaultdict
 from json import dumps, loads
@@ -607,17 +627,12 @@ space-MNI152NLin6Asym_desc-preproc_bold.json'
                             (4, 4) if self.inputs.space in STANDARD_SPACES else (2, 2)
                         )
 
-                    curr_zooms = zooms = hdr.get_zooms()
-                    if "RepetitionTime" in self.inputs.get():
-                        zooms = curr_zooms[:3] + (self.inputs.RepetitionTime,)
-
-                    if (curr_codes, curr_units, curr_zooms) != (xcodes, units, zooms):
+                    if curr_codes != xcodes or curr_units != units:
                         self._results["fixed_hdr"][i] = True
                         new_header = hdr.copy()
                         new_header.set_qform(nii.affine, xcodes[0])
                         new_header.set_sform(nii.affine, xcodes[1])
                         new_header.set_xyzt_units(*units)
-                        new_header.set_zooms(zooms)
 
                 if data_dtype == "source":  # match source dtype
                     try:
