@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """Plotting tools shared across MRIQC and FMRIPREP."""
 
 import numpy as np
@@ -17,8 +37,6 @@ from nilearn.signal import clean
 from nilearn._utils import check_niimg_4d
 from nilearn._utils.niimg import _safe_get_data
 
-import seaborn as sns
-from seaborn import color_palette
 
 DINA4_LANDSCAPE = (11.69, 8.27)
 
@@ -83,6 +101,8 @@ class fMRIPlot:
 
     def plot(self, figure=None):
         """Main plotter"""
+        import seaborn as sns
+
         sns.set_style("whitegrid")
         sns.set_context("paper", font_scale=0.8)
 
@@ -106,6 +126,8 @@ class fMRIPlot:
             grid_id += 1
 
         if self.confounds:
+            from seaborn import color_palette
+
             palette = color_palette("husl", nconfounds)
 
         for i, (name, kwargs) in enumerate(self.confounds.items()):
@@ -582,6 +604,7 @@ def confoundplot(
     cutoff=None,
     ylims=None,
 ):
+    import seaborn as sns
 
     # Define TR and number of frames
     notr = False
@@ -903,40 +926,57 @@ def compcor_variance_plot(
 
 
 def confounds_correlation_plot(
-    confounds_file, output_file=None, figure=None, reference="global_signal", max_dim=70
+    confounds_file,
+    columns=None,
+    figure=None,
+    max_dim=20,
+    output_file=None,
+    reference="global_signal",
 ):
     """
+    Generate a bar plot with the correlation of confounds.
+
     Parameters
     ----------
-    confounds_file: str
+    confounds_file: :obj:`str`
         File containing all confound regressors to be included in the
         correlation plot.
-    output_file: str or None
-        Path where the output figure should be saved. If this is not defined,
-        then the plotting axes will be returned instead of the saved figure
-        path.
     figure: figure or None
         Existing figure on which to plot.
-    reference: str
-        `confounds_correlation_plot` prepares a bar plot of the correlations
-        of each confound regressor with a reference column. By default, this
-        is the global signal (so that collinearities with the global signal
-        can readily be assessed).
-    max_dim: int
+    columns: :obj:`list` or :obj:`None`.
+        Select a list of columns from the dataset.
+    max_dim: :obj:`int`
         The maximum number of regressors to be included in the output plot.
         Reductions (e.g., CompCor) of high-dimensional data can yield so many
         regressors that the correlation structure becomes obfuscated. This
-        criterion selects the `max_dim` regressors that have the largest
-        correlation magnitude with `reference` for inclusion in the plot.
+        criterion selects the ``max_dim`` regressors that have the largest
+        correlation magnitude with ``reference`` for inclusion in the plot.
+    output_file: :obj:`str` or :obj:`None`
+        Path where the output figure should be saved. If this is not defined,
+        then the plotting axes will be returned instead of the saved figure
+        path.
+    reference: :obj:`str`
+        ``confounds_correlation_plot`` prepares a bar plot of the correlations
+        of each confound regressor with a reference column. By default, this
+        is the global signal (so that collinearities with the global signal
+        can readily be assessed).
 
     Returns
     -------
     axes and gridspec
-        Plotting axes and gridspec. Returned only if `output_file` is None.
-    output_file: str
+        Plotting axes and gridspec. Returned only if ``output_file`` is ``None``.
+    output_file: :obj:`str`
         The file where the figure is saved.
     """
+    import seaborn as sns
+
     confounds_data = pd.read_table(confounds_file)
+
+    if columns:
+        columns = set(columns)  # Drop duplicates
+        columns.add(reference)  # Make sure the reference is included
+        confounds_data = confounds_data[[el for el in columns]]
+
     confounds_data = confounds_data.loc[
         :, np.logical_not(np.isclose(confounds_data.var(skipna=True), 0))
     ]
