@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """Helper tools for visualization purposes."""
 from pathlib import Path
 from shutil import which
@@ -15,8 +35,6 @@ import nibabel as nb
 
 from nilearn import image as nlimage
 from nilearn.plotting import plot_anat
-from svgutils.transform import fromstring, SVGFigure, GroupElement
-from seaborn import color_palette
 
 from nipype.utils import filemanip
 from .. import NIWORKFLOWS_LOG
@@ -134,7 +152,7 @@ def extract_svg(display_object, dpi=300, compress="auto"):
     start_idx = image_svg.find(start_tag)
     end_tag = "</svg>"
     end_idx = image_svg.rfind(end_tag)
-    if start_idx is -1 or end_idx is -1:
+    if start_idx == -1 or end_idx == -1:
         NIWORKFLOWS_LOG.info("svg tags not found in extract_svg")
     # rfind gives the start index of the substr. We want this substr
     # included in our return value so we add its length to the index.
@@ -225,6 +243,8 @@ def plot_segs(
     coordinates. plot_params will be passed on to nilearn plot_* functions. If
     seg_niis is a list of size one, it behaves as if it was plotting the mask.
     """
+    from svgutils.transform import fromstring
+
     plot_params = {} if plot_params is None else plot_params
 
     image_nii = _3d_in_file(image_nii)
@@ -268,6 +288,8 @@ def _plot_anat_with_contours(image, segs=None, compress="auto", **plot_params):
     levels = plot_params.pop("levels", None) or []
     missing = nsegs - len(colors)
     if missing > 0:  # missing may be negative
+        from seaborn import color_palette
+
         colors = colors + color_palette("husl", missing)
 
     colors = [[c] if not isinstance(c, list) else c for c in colors]
@@ -302,11 +324,14 @@ def plot_registration(
     label=None,
     contour=None,
     compress="auto",
+    dismiss_affine=False,
 ):
     """
     Plots the foreground and background views
     Default order is: axial, coronal, sagittal
     """
+    from svgutils.transform import fromstring
+
     plot_params = {} if plot_params is None else plot_params
 
     # Use default MNI cuts if none defined
@@ -327,6 +352,15 @@ def plot_registration(
         white = nlimage.new_img_like(contour, contour_data == 2)
         pial = nlimage.new_img_like(contour, contour_data >= 2)
 
+    if dismiss_affine:
+        canonical_r = rotation2canonical(anat_nii)
+        anat_nii = rotate_affine(anat_nii, rot=canonical_r)
+        if ribbon:
+            white = rotate_affine(white, rot=canonical_r)
+            pial = rotate_affine(pial, rot=canonical_r)
+        if contour:
+            contour = rotate_affine(contour, rot=canonical_r)
+
     # Plot each cut axis
     for i, mode in enumerate(list(order)):
         plot_params["display_mode"] = mode
@@ -343,7 +377,7 @@ def plot_registration(
             display.add_contours(white, colors="b", **kwargs)
             display.add_contours(pial, colors="r", **kwargs)
         elif contour is not None:
-            display.add_contours(contour, colors="b", levels=[0.5], linewidths=0.5)
+            display.add_contours(contour, colors="r", levels=[0.5], linewidths=0.5)
 
         svg = extract_svg(display, compress=compress)
         display.close()
@@ -363,6 +397,8 @@ def compose_view(bg_svgs, fg_svgs, ref=0, out_file="report.svg"):
 
 
 def _compose_view(bg_svgs, fg_svgs, ref=0):
+    from svgutils.compose import Unit
+    from svgutils.transform import SVGFigure, GroupElement
 
     if fg_svgs is None:
         fg_svgs = []
@@ -389,11 +425,11 @@ def _compose_view(bg_svgs, fg_svgs, ref=0):
 
     # Compose the views panel: total size is the width of
     # any element (used the first here) and the sum of heights
-    fig = SVGFigure(width, heights[:nsvgs].sum())
+    fig = SVGFigure(Unit(f"{width}px"), Unit(f"{heights[:nsvgs].sum()}px"))
 
     yoffset = 0
     for i, r in enumerate(roots):
-        r.moveto(0, yoffset, scale=scales[i])
+        r.moveto(0, yoffset, scale_x=scales[i])
         if i == (nsvgs - 1):
             yoffset = 0
         else:
@@ -408,8 +444,8 @@ def _compose_view(bg_svgs, fg_svgs, ref=0):
     else:
         newroots = roots
     fig.append(newroots)
-    fig.root.attrib.pop("width")
-    fig.root.attrib.pop("height")
+    fig.root.attrib.pop("width", None)
+    fig.root.attrib.pop("height", None)
     fig.root.set("preserveAspectRatio", "xMidYMid meet")
 
     with TemporaryDirectory() as tmpdirname:
